@@ -1,36 +1,23 @@
 /*
-*   Copyright (C) 2017 by Marco Martin <mart@kde.org>
-*
-*   This program is free software; you can redistribute it and/or modify
-*   it under the terms of the GNU Library General Public License as
-*   published by the Free Software Foundation; either version 2, or
-*   (at your option) any later version.
-*
-*   This program is distributed in the hope that it will be useful,
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-*   GNU Library General Public License for more details
-*
-*   You should have received a copy of the GNU Library General Public
-*   License along with this program; if not, write to the
-*   Free Software Foundation, Inc.,
-*   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ *  SPDX-FileCopyrightText: 2017 by Marco Martin <mart@kde.org>
+ *
+ *  SPDX-License-Identifier: LGPL-2.0-or-later
+ */
 
 #ifndef PLATFORMTHEME_H
 #define PLATFORMTHEME_H
 
-#include <QObject>
-#include <QQuickItem>
 #include <QColor>
+#include <QIcon>
+#include <QObject>
 #include <QPalette>
+#include <QQuickItem>
 
-#ifndef KIRIGAMI_BUILD_TYPE_STATIC
 #include <kirigami2_export.h>
-#endif
-
-namespace Kirigami {
 
+namespace Kirigami
+{
+class PlatformThemeData;
 class PlatformThemePrivate;
 
 /**
@@ -40,11 +27,7 @@ class PlatformThemePrivate;
  * different platforms can reimplement this class to integrate with
  * system platform colors of a given platform
  */
-#ifdef KIRIGAMI_BUILD_TYPE_STATIC
-class PlatformTheme : public QObject
-#else
 class KIRIGAMI2_EXPORT PlatformTheme : public QObject
-#endif
 {
     Q_OBJECT
 
@@ -86,7 +69,8 @@ class KIRIGAMI2_EXPORT PlatformTheme : public QObject
     /**
      * Color for text that has been highlighted, often is a light color while normal text is dark
      */
-    Q_PROPERTY(QColor highlightedTextColor READ highlightedTextColor WRITE setCustomHighlightedTextColor RESET setCustomHighlightedTextColor NOTIFY colorsChanged)
+    Q_PROPERTY(
+        QColor highlightedTextColor READ highlightedTextColor WRITE setCustomHighlightedTextColor RESET setCustomHighlightedTextColor NOTIFY colorsChanged)
 
     /**
      * Foreground for areas that are active or requesting attention
@@ -118,18 +102,62 @@ class KIRIGAMI2_EXPORT PlatformTheme : public QObject
      */
     Q_PROPERTY(QColor positiveTextColor READ positiveTextColor WRITE setCustomPositiveTextColor RESET setCustomPositiveTextColor NOTIFY colorsChanged)
 
-    //background colors
+    // background colors
     /**
      * The generic background color
      */
     Q_PROPERTY(QColor backgroundColor READ backgroundColor WRITE setCustomBackgroundColor RESET setCustomBackgroundColor NOTIFY colorsChanged)
 
+    /**
+     * The generic background color
+     * Alternate background; for example, for use in lists.
+     * This color may be the same as BackgroundNormal,
+     * especially in sets other than View and Window.
+     */
+    Q_PROPERTY(QColor alternateBackgroundColor READ alternateBackgroundColor WRITE setCustomAlternateBackgroundColor RESET setCustomAlternateBackgroundColor
+                   NOTIFY colorsChanged)
+
     /**
      * The background color for selected areas
      */
     Q_PROPERTY(QColor highlightColor READ highlightColor WRITE setCustomHighlightColor RESET setCustomHighlightColor NOTIFY colorsChanged)
 
-    //decoration colors
+    /**
+     * Background for areas that are active or requesting attention
+     */
+    Q_PROPERTY(
+        QColor activeBackgroundColor READ activeBackgroundColor WRITE setCustomActiveBackgroundColor RESET setCustomActiveBackgroundColor NOTIFY colorsChanged)
+
+    /**
+     * Background color for links
+     */
+    Q_PROPERTY(QColor linkBackgroundColor READ linkBackgroundColor WRITE setCustomLinkBackgroundColor RESET setCustomLinkBackgroundColor NOTIFY colorsChanged)
+
+    /**
+     * Background color for visited links, usually a bit darker than linkBackgroundColor
+     */
+    Q_PROPERTY(QColor visitedLinkBackgroundColor READ visitedLinkBackgroundColor WRITE setCustomVisitedLinkBackgroundColor RESET
+                   setCustomVisitedLinkBackgroundColor NOTIFY colorsChanged)
+
+    /**
+     * Background color for negative areas, such as critical errors and destructive actions
+     */
+    Q_PROPERTY(QColor negativeBackgroundColor READ negativeBackgroundColor WRITE setCustomNegativeBackgroundColor RESET setCustomNegativeBackgroundColor NOTIFY
+                   colorsChanged)
+
+    /**
+     * Background color for neutral areas, such as warnings (but not critical)
+     */
+    Q_PROPERTY(QColor neutralBackgroundColor READ neutralBackgroundColor WRITE setCustomNeutralBackgroundColor RESET setCustomNeutralBackgroundColor NOTIFY
+                   colorsChanged)
+
+    /**
+     * Background color for positive areas, such as success messages and trusted content
+     */
+    Q_PROPERTY(QColor positiveBackgroundColor READ positiveBackgroundColor WRITE setCustomPositiveBackgroundColor RESET setCustomPositiveBackgroundColor NOTIFY
+                   colorsChanged)
+
+    // decoration colors
     /**
      * A decoration color that indicates active focus
      */
@@ -142,18 +170,24 @@ class KIRIGAMI2_EXPORT PlatformTheme : public QObject
 
     // font and palette
     Q_PROPERTY(QFont defaultFont READ defaultFont NOTIFY defaultFontChanged)
-    //Active palette
+
+    // small font
+    Q_PROPERTY(QFont smallFont READ smallFont NOTIFY defaultFontChanged)
+
+    // Active palette
     Q_PROPERTY(QPalette palette READ palette NOTIFY paletteChanged)
 
 public:
-
     enum ColorSet {
         View = 0, /** Color set for item views, usually the lightest of all */
         Window, /** Default Color set for windows and "chrome" areas */
         Button, /** Color set used by buttons */
         Selection, /** Color set used by selectged areas */
         Tooltip, /** Color set used by tooltips */
-        Complementary /** Color set meant to be complementary to Window: usually is a dark theme for light themes */
+        Complementary, /** Color set meant to be complementary to Window: usually is a dark theme for light themes */
+        Header, /** Color set to be used by heading areas of applications, such as toolbars */
+
+        ColorSetCount, // Number of items in this enum, this should always be the last item.
     };
     Q_ENUM(ColorSet)
 
@@ -161,12 +195,14 @@ public:
         Disabled = QPalette::Disabled,
         Active = QPalette::Active,
         Inactive = QPalette::Inactive,
-        Normal = QPalette::Normal
+        Normal = QPalette::Normal,
+
+        ColorGroupCount, // Number of items in this enum, this should always be the last item.
     };
     Q_ENUM(ColorGroup)
 
-    explicit PlatformTheme(QObject *parent = 0);
-    ~PlatformTheme();
+    explicit PlatformTheme(QObject *parent = nullptr);
+    ~PlatformTheme() override;
 
     void setColorSet(PlatformTheme::ColorSet);
     PlatformTheme::ColorSet colorSet() const;
@@ -177,7 +213,7 @@ public:
     bool inherit() const;
     void setInherit(bool inherit);
 
-    //foreground colors
+    // foreground colors
     QColor textColor() const;
     QColor disabledTextColor() const;
     QColor highlightedTextColor() const;
@@ -188,24 +224,33 @@ public:
     QColor neutralTextColor() const;
     QColor positiveTextColor() const;
 
-    //background colors
+    // background colors
     QColor backgroundColor() const;
+    QColor alternateBackgroundColor() const;
     QColor highlightColor() const;
-    //TODO: add active/positive/neutral/negative backgrounds?
+    QColor activeBackgroundColor() const;
+    QColor linkBackgroundColor() const;
+    QColor visitedLinkBackgroundColor() const;
+    QColor negativeBackgroundColor() const;
+    QColor neutralBackgroundColor() const;
+    QColor positiveBackgroundColor() const;
 
-    //decoration colors
+    // decoration colors
     QColor focusColor() const;
     QColor hoverColor() const;
 
     QFont defaultFont() const;
+    QFont smallFont() const;
 
-    //this may is used by the desktop QQC2 to set the styleoption palettes
+    // this may is used by the desktop QQC2 to set the styleoption palettes
     QPalette palette() const;
 
-    //this will be used by desktopicon to fetch icons with KIconLoader
+    // this will be used by desktopicon to fetch icons with KIconLoader
     virtual Q_INVOKABLE QIcon iconFromTheme(const QString &name, const QColor &customColor = Qt::transparent);
 
-    //foreground colors
+    bool supportsIconColoring() const;
+
+    // foreground colors
     void setCustomTextColor(const QColor &color = QColor());
     void setCustomDisabledTextColor(const QColor &color = QColor());
     void setCustomHighlightedTextColor(const QColor &color = QColor());
@@ -215,29 +260,38 @@ public:
     void setCustomNegativeTextColor(const QColor &color = QColor());
     void setCustomNeutralTextColor(const QColor &color = QColor());
     void setCustomPositiveTextColor(const QColor &color = QColor());
-    //background colors
+    // background colors
     void setCustomBackgroundColor(const QColor &color = QColor());
+    void setCustomAlternateBackgroundColor(const QColor &color = QColor());
     void setCustomHighlightColor(const QColor &color = QColor());
-    //decoration colors
+    void setCustomActiveBackgroundColor(const QColor &color = QColor());
+    void setCustomLinkBackgroundColor(const QColor &color = QColor());
+    void setCustomVisitedLinkBackgroundColor(const QColor &color = QColor());
+    void setCustomNegativeBackgroundColor(const QColor &color = QColor());
+    void setCustomNeutralBackgroundColor(const QColor &color = QColor());
+    void setCustomPositiveBackgroundColor(const QColor &color = QColor());
+    // decoration colors
     void setCustomFocusColor(const QColor &color = QColor());
     void setCustomHoverColor(const QColor &color = QColor());
 
-    //QML attached property
+    // QML attached property
     static PlatformTheme *qmlAttachedProperties(QObject *object);
 
 Q_SIGNALS:
-    //TODO: parameters to signals as this is also a c++ api
+    // TODO: parameters to signals as this is also a c++ api
     void colorsChanged();
     void defaultFontChanged(const QFont &font);
+    void smallFontChanged(const QFont &font);
     void colorSetChanged(Kirigami::PlatformTheme::ColorSet colorSet);
     void colorGroupChanged(Kirigami::PlatformTheme::ColorGroup colorGroup);
     void paletteChanged(const QPalette &pal);
     void inheritChanged(bool inherit);
 
 protected:
-    //Setters, not accessible from QML but from implementations
-    
-    //foreground colors
+    // Setters, not accessible from QML but from implementations
+    void setSupportsIconColoring(bool support);
+
+    // foreground colors
     void setTextColor(const QColor &color);
     void setDisabledTextColor(const QColor &color);
     void setHighlightedTextColor(const QColor &color);
@@ -248,23 +302,84 @@ protected:
     void setNeutralTextColor(const QColor &color);
     void setPositiveTextColor(const QColor &color);
 
-    //background colors
+    // background colors
     void setBackgroundColor(const QColor &color);
+    void setAlternateBackgroundColor(const QColor &color);
     void setHighlightColor(const QColor &color);
+    void setActiveBackgroundColor(const QColor &color);
+    void setLinkBackgroundColor(const QColor &color);
+    void setVisitedLinkBackgroundColor(const QColor &color);
+    void setNegativeBackgroundColor(const QColor &color);
+    void setNeutralBackgroundColor(const QColor &color);
+    void setPositiveBackgroundColor(const QColor &color);
 
-    //decoration colors
+    // decoration colors
     void setFocusColor(const QColor &color);
     void setHoverColor(const QColor &color);
 
     void setDefaultFont(const QFont &defaultFont);
+    void setSmallFont(const QFont &smallFont);
+
+#if KIRIGAMI2_ENABLE_DEPRECATED_SINCE(5, 80)
+    KIRIGAMI2_DEPRECATED_VERSION(5, 80, "setPalette is unused, the palette is autogenerated from PlatformTheme colors now")
     void setPalette(const QPalette &palette);
+#endif
+
+    bool event(QEvent *event) override;
+
 private:
+    void update();
+    void updateChildren(QObject *item);
+    void emitSignals();
+    void emitColorChanged();
+    QObject *determineParent(QObject *object);
+
     PlatformThemePrivate *d;
     friend class PlatformThemePrivate;
+    friend class PlatformThemeData;
 };
 
+namespace PlatformThemeEvents
+{
+// To avoid the overhead of Qt's signal/slot connections, we use custom events
+// to communicate with subclasses. This way, we can indicate what actually
+// changed without needing to add new virtual functions to PlatformTheme which
+// would break binary compatibility.
+//
+// To handle these events in your subclass, override QObject::event() and check
+// if you receive one of these events, then do what is needed. Finally, make
+// sure to call PlatformTheme::event() since that will also do some processing
+// of these events.
+
+template<typename T>
+class KIRIGAMI2_EXPORT PropertyChangedEvent : public QEvent
+{
+public:
+    PropertyChangedEvent(PlatformTheme *theme, const T &previous, const T &current)
+        : QEvent(PropertyChangedEvent<T>::type)
+        , sender(theme)
+        , oldValue(previous)
+        , newValue(current)
+    {
+    }
+
+    PlatformTheme *sender;
+    T oldValue;
+    T newValue;
+
+    static QEvent::Type type;
+};
+
+using DataChangedEvent = PropertyChangedEvent<std::shared_ptr<PlatformThemeData>>;
+using ColorSetChangedEvent = PropertyChangedEvent<PlatformTheme::ColorSet>;
+using ColorGroupChangedEvent = PropertyChangedEvent<PlatformTheme::ColorGroup>;
+using ColorChangedEvent = PropertyChangedEvent<QColor>;
+using FontChangedEvent = PropertyChangedEvent<QFont>;
+
 }
 
+} // namespace Kirigami
+
 QML_DECLARE_TYPEINFO(Kirigami::PlatformTheme, QML_HAS_ATTACHED_PROPERTIES)
 
 #endif // PLATFORMTHEME_H
